package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ListBindingImpl;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelElementList;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.jboss.tools.sapphire.IComposite;
import org.jboss.tools.sapphire.IService;

public final class Composite
    
    extends ModelElement
    implements IComposite
    
{
    private ModelElementList<IService> services;
    
    public Composite( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public Composite( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public ModelElementList<IService> getServices()
    {
        synchronized( root() )
        {
            if( this.services == null )
            {
                refresh( PROP_SERVICES, true );
            }
            
            return this.services;
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_SERVICES )
            {
                if( this.services == null )
                {
                    if( force == true )
                    {
                        this.services = new ModelElementList<IService>( this, PROP_SERVICES );
                        final ListBindingImpl binding = resource().binding( PROP_SERVICES );
                        this.services.init( binding );
                        refreshPropertyEnabledStatus( PROP_SERVICES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_SERVICES );
                    final boolean notified = this.services.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_SERVICES );
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_SERVICES )
        {
            return getServices();
        }
        
        return super.read( property );
    }
    
}
