package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ListBindingImpl;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelElementList;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.jboss.tools.sapphire.IComposite;
import org.jboss.tools.sapphire.ISwitchyard;

public final class Switchyard
    
    extends ModelElement
    implements ISwitchyard
    
{
    private ModelElementList<IComposite> composites;
    
    public Switchyard( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public Switchyard( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public ModelElementList<IComposite> getComposites()
    {
        synchronized( root() )
        {
            if( this.composites == null )
            {
                refresh( PROP_COMPOSITES, true );
            }
            
            return this.composites;
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_COMPOSITES )
            {
                if( this.composites == null )
                {
                    if( force == true )
                    {
                        this.composites = new ModelElementList<IComposite>( this, PROP_COMPOSITES );
                        final ListBindingImpl binding = resource().binding( PROP_COMPOSITES );
                        this.composites.init( binding );
                        refreshPropertyEnabledStatus( PROP_COMPOSITES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_COMPOSITES );
                    final boolean notified = this.composites.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_COMPOSITES );
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_COMPOSITES )
        {
            return getComposites();
        }
        
        return super.read( property );
    }
    
}
